@@ -34,11 +34,12 @@
 #ifndef SQUID_HTTPHIERARCHYLOGENTRY_H
 #define SQUID_HTTPHIERARCHYLOGENTRY_H
 
-#include "ip/IpAddress.h"
+#include "hier_code.h"
+#include "lookup_t.h"
 #include "rfc2181.h"
 #include "PingData.h"
 
-/** todo Cleanup: break hier_code type out. We don't need the rest. */
+/* for http_status */
 #include "enums.h"
 
 class HierarchyLogEntry
@@ -63,7 +64,7 @@ public:
     int64_t peer_response_time; ///< last peer response delay
     timeval first_conn_start; ///< first connection use among all peers
     int64_t total_response_time; ///< cumulative for all peers
-    IpAddress peer_local_addr; ///< local IP:port of the last server-side connection
+    u_short peer_local_port; //< local port of the last server-side connection
 };
 
 extern void hierarchyNote(HierarchyLogEntry *, hier_code, const char *);
