@@ -10,7 +10,7 @@
 #include "adaptation/AccessRule.h"
 #include "adaptation/Config.h"
 #include "adaptation/AccessCheck.h"
-
+#include "base/TextException.h"
 
 /** \cond AUTODOCS-IGNORE */
 cbdata_type Adaptation::AccessCheck::CBDATA_AccessCheck = CBDATA_UNKNOWN;
@@ -23,9 +23,8 @@ Adaptation::AccessCheck::Start(Method method, VectPoint vp,
 
     if (Config::Enabled) {
         // the new check will call the callback and delete self, eventually
-        AsyncJob::Start(new AccessCheck( // we do not store so not a CbcPointer
-                            ServiceFilter(method, vp, req, rep), cb, cbdata));
-        return true;
+        return AsyncStart(new AccessCheck(
+                              ServiceFilter(method, vp, req, rep), cb, cbdata));
     }
 
     debugs(83, 3, HERE << "adaptation off, skipping");
@@ -124,15 +123,7 @@ Adaptation::AccessCheck::AccessCheckCallbackWrapper(int answer, void *data)
      * we should be kicking off an authentication before continuing
      * with this request. see bug 2400 for details.
      */
-
-    // convert to async call to get async call protections and features
-    typedef UnaryMemFunT<AccessCheck, int> MyDialer;
-    AsyncCall::Pointer call =
-        asyncCall(93,7, "Adaptation::AccessCheck::noteAnswer",
-                  MyDialer(ac, &Adaptation::AccessCheck::noteAnswer,
-                           answer==ACCESS_ALLOWED));
-    ScheduleCallHere(call);
-
+    ac->noteAnswer(answer==ACCESS_ALLOWED);
 }
 
 /// process the results of the ACL check
