@@ -4,9 +4,9 @@
 #include "squid.h"
 #include <list>
 #include <libecap/adapter/service.h>
-#include "TextException.h"
 #include "adaptation/ecap/ServiceRep.h"
 #include "adaptation/ecap/XactionRep.h"
+#include "base/TextException.h"
 
 // configured eCAP service wrappers
 static std::list<Adaptation::Ecap::ServiceRep::AdapterService> TheServices;
@@ -57,11 +57,11 @@ bool Adaptation::Ecap::ServiceRep::wantsUrl(const String &urlPath) const
 }
 
 Adaptation::Initiate *
-Adaptation::Ecap::ServiceRep::makeXactLauncher(HttpMsg *virgin,
-        HttpRequest *cause)
+Adaptation::Ecap::ServiceRep::makeXactLauncher(Adaptation::Initiator *initiator,
+        HttpMsg *virgin, HttpRequest *cause)
 {
     Must(up());
-    XactionRep *rep = new XactionRep(virgin, cause, Pointer(this));
+    XactionRep *rep = new XactionRep(initiator, virgin, cause, Pointer(this));
     XactionRep::AdapterXaction x(theService->makeXaction(rep));
     rep->master(x);
     return rep;
