@@ -39,6 +39,7 @@
 #include "SquidMath.h"
 #include "SquidTime.h"
 #include "SwapDir.h"
+#include "swap_log_op.h"
 
 #if HAVE_STATVFS
 #if HAVE_SYS_STATVFS_H
@@ -343,8 +344,8 @@ StoreController::updateSize(int64_t size, int sign)
 void
 SwapDir::updateSize(int64_t size, int sign)
 {
-    int64_t blks = (size + fs.blksize - 1) / fs.blksize;
-    int64_t k = ((blks * fs.blksize) >> 10) * sign;
+    int blks = (size + fs.blksize - 1) / fs.blksize;
+    int k = (blks * fs.blksize >> 10) * sign;
     cur_size += k;
     store_swap_size += k;
 
@@ -360,13 +361,13 @@ StoreController::stat(StoreEntry &output) const
     storeAppendPrintf(&output, "Store Directory Statistics:\n");
     storeAppendPrintf(&output, "Store Entries          : %lu\n",
                       (unsigned long int)StoreEntry::inUseCount());
-    storeAppendPrintf(&output, "Maximum Swap Size      : %" PRIu64 " KB\n",
-                      maxSize());
+    storeAppendPrintf(&output, "Maximum Swap Size      : %8ld KB\n",
+                      (long int) maxSize());
     storeAppendPrintf(&output, "Current Store Swap Size: %8lu KB\n",
                       store_swap_size);
-    storeAppendPrintf(&output, "Current Capacity       : %" PRId64 "%% used, %" PRId64 "%% free\n",
-                      Math::int64Percent(store_swap_size, maxSize()),
-                      Math::int64Percent((maxSize() - store_swap_size), maxSize()));
+    storeAppendPrintf(&output, "Current Capacity       : %d%% used, %d%% free\n",
+                      Math::intPercent((int) store_swap_size, (int) maxSize()),
+                      Math::intPercent((int) (maxSize() - store_swap_size), (int) maxSize()));
     /* FIXME Here we should output memory statistics */
 
     /* now the swapDir */
@@ -374,14 +375,14 @@ StoreController::stat(StoreEntry &output) const
 }
 
 /* if needed, this could be taught to cache the result */
-uint64_t
+size_t
 StoreController::maxSize() const
 {
     /* TODO: include memory cache ? */
     return swapDir->maxSize();
 }
 
-uint64_t
+size_t
 StoreController::minSize() const
 {
     /* TODO: include memory cache ? */
@@ -827,21 +828,22 @@ StoreHashIndex::init()
     }
 }
 
-uint64_t
+size_t
 StoreHashIndex::maxSize() const
 {
-    uint64_t result = 0;
+    int i;
+    size_t result = 0;
 
-    for (int i = 0; i < Config.cacheSwap.n_configured; i++)
+    for (i = 0; i < Config.cacheSwap.n_configured; i++)
         result += store(i)->maxSize();
 
     return result;
 }
 
-uint64_t
+size_t
 StoreHashIndex::minSize() const
 {
-    uint64_t result = 0;
+    size_t result = 0;
 
     for (int i = 0; i < Config.cacheSwap.n_configured; i++)
         result += store(i)->minSize();
